/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.classfile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.support.WrapperMember;
import org.apache.bcel.support.WrapperRecycler;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.MethodNode;

/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public final class ClassParser {

	/**
	 * Wrappee field.
	 */
	@WrapperMember
	private ClassReader reader = null; 
	
	/**
	 * Wrapper constructor.
	 *
	 * @param node
	 */
	@WrapperMember
	ClassParser(ClassReader reader) {
		setReader(reader);
	}

	/**
	 * Sets the class node wrappee, registering in the identity map.
	 *
	 * @param reader
	 */
	@WrapperMember
	protected ClassReader setReader(ClassReader reader) {
		this.reader = reader;
		WrapperRecycler.add(this, reader);
		return reader;
	}
	
	/**
	 * Getter allows for wrappee refinement.
	 *
	 * @return
	 */
	@WrapperMember
	protected ClassReader getReader() {
		return reader;
	}

	/**
	 * Creates a parser to read from this file, which has this name.
	 *
	 * @param file
	 * @param file_name
	 */
	public ClassParser(InputStream file, String file_name) {
		try {
			setReader(new ClassReader(file));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Parses the file and generates a JavaClass.
	 *
	 * @return
	 * @throws IOException
	 * @throws ClassFormatException
	 */
	public JavaClass parse() throws IOException, ClassFormatException {
		// Parse the class
		ClassNode node = new ClassNode();
		getReader().accept(node, 0);
		
		// make sure that the handle/instruction structure is created for each method
		
		// for each method
		List<MethodNode> newMethods = new ArrayList<MethodNode>();
		ListIterator<?> methodIterator = node.methods.listIterator();
		while(methodIterator.hasNext()) {
			MethodNode m = (MethodNode) methodIterator.next();
		
			// by cloning the method we create the needed structure around it
			newMethods.add(MethodGen.clone(m, false));			
		}
		
		node.methods = newMethods;
		
		// Wrap into a JavaClass
		return new JavaClass(node);
	}
}
