/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import java.util.LinkedList;
import java.util.Queue;

import org.apache.bcel.support.WrapperMember;
import org.apache.bcel.support.WrapperRecycler;
import org.objectweb.asm.tree.AbstractInsnNode;


/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public final class BranchHandle extends InstructionHandle {
	

	private static Queue<BranchHandle> pool = new LinkedList<BranchHandle>();
	
	/**
	 * Wrapper constructor.
	 *
	 * @param node
	 */
	@WrapperMember
	BranchHandle(AbstractInsnNode node) {
		super(node);
	}
	
	static BranchHandle createHandle(AbstractInsnNode node) {
		if (! pool.isEmpty()) {
			BranchHandle handle = pool.poll();
			handle.setNode(node);
			return handle;
		}
		return new BranchHandle(node);
	}
	
	public void dispose() {
		AbstractInsnNode node = getNode();
		WrapperRecycler.remove(instruction, node);
		WrapperRecycler.remove(this, node);
		this.instruction.removeHandle(this);
		this.setNode(null);
		this.instruction = null;
		this.list = null;
		this.targeters = null;
		this.setPosition(-1);
		pool.add(this);
	}
	
	/**
	 * Sets a new target for this branch instruction.
	 *
	 * @param handle
	 */
	public void setTarget(InstructionHandle handle) {
		assert(handle.list == this.list);
		
		BranchInstruction bi = (BranchInstruction) getInstruction();
		bi.setTarget(this, handle);
	}

	public int getPosition() {
		return super.getPosition();
	}
}
