/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import org.apache.bcel.classfile.AccessFlags;
import org.apache.bcel.support.PushedDown;
import org.apache.bcel.support.WrapperMember;
import org.apache.bcel.support.WrapperRecycler;
import org.objectweb.asm.tree.MemberNode;

/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public abstract class FieldGenOrMethodGen extends AccessFlags implements NamedAndTyped {
	
	/**
	 * The wrappee. Should be refined by sub-types.
	 */
	@WrapperMember
	private MemberNode node;
	
	/**
	 * Wrapper constructor.
	 *
	 * @param node
	 */
	@WrapperMember
	FieldGenOrMethodGen(MemberNode node) {
		setMemberNode(node);
	}

	/**
	 * Sets the class node wrappee, registering in the identity map.
	 *
	 * @param node
	 */
	@WrapperMember
	protected MemberNode setMemberNode(MemberNode node) {
		this.node = node;
		WrapperRecycler.add(this, node);
		return node;
	}
	
	/**
	 * Getter allows for wrappee refinement.
	 *
	 * @return
	 */
	@WrapperMember
	protected MemberNode getMemberNode() {
		return node;
	}
	
	/**
	 * Empty constructor.
	 */
	protected FieldGenOrMethodGen() {
	}

	/**
	 * Force sub-types to implement this method since the
	 * wrappee does not have a name property.
	 *
	 * @return
	 */
	@PushedDown
	public abstract String getName();
}
