/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import org.apache.bcel.support.WrapperMember;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;

/**
 * 
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public class IFNE extends IfInstruction {
	
	public IFNE(InstructionHandle handle) {
		setNode(new JumpInsnNode(Opcodes.IFNE, handle.ensureLabel()));
		registerTargeter(handle);
	}
	
	/**
	 * Getter allows for wrappee refinement.
	 *
	 * @return
	 */
	@WrapperMember
	protected JumpInsnNode getNode() {
		return (JumpInsnNode) super.getNode();
	}
	
	/**
	 * Gets the label that is the target of this branch.
	 *
	 * @param label
	 */
	@WrapperMember
	LabelNode getTargetLabel() {
		return getNode().label;
	}
	
	/**
	 * Sets the label that is the target of this branch.
	 *
	 * @param label
	 */
	@WrapperMember
	void setTargetLabel(AbstractInsnNode node, LabelNode label) {
		((JumpInsnNode) node).label = label;
	}
	
	@WrapperMember
	IFNE() {	
	}
	
	/**
	 * Shallow clone of this instruction.
	 *
	 * @return
	 */
	@WrapperMember
	public Instruction cloneInstruction() {
		IFNE g = new IFNE();
		g.setNode(this.cloneNode());
		return g;
	}
	
	@Override
	@WrapperMember
	public AbstractInsnNode cloneNode() {
		return new JumpInsnNode(Opcodes.IFNE, ((JumpInsnNode) getNode()).label);
	}
	
	/**
	 * Returns all labels that are targets of this instruction.
	 *
	 * @return
	 */
	@WrapperMember
	public LabelNode[] getTargetLabels() {
		return new LabelNode[] {((JumpInsnNode) getNode()).label};
	}
}
