/**
 * Copyright (c) 2011 Generative Software Development Lab, 
 * University of Waterloo, Canada. 
 * All Rights Reserved.
 * ----------------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m  
 */
package org.apache.bcel.generic;

import org.apache.bcel.support.WrapperMember;
import org.apache.bcel.support.WrapperRecycler;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.LocalVariableNode;

/**
 * TODO Comment!!
 * 
 * @author Thiago Tonelli Bartolomei <ttonelli@uwaterloo.ca>
 */
public class LocalVariableGen implements InstructionTargeter {
	
	/**
	 * Wrappee field.
	 */
	@WrapperMember
	private LocalVariableNode node = null;
	
	/**
	 * Wrapper constructor.
	 *
	 * @param node
	 */
	@WrapperMember
	LocalVariableGen(LocalVariableNode node) {
		setNode(node);
	}

	/**
	 * Sets the class node wrappee, registering in the identity map.
	 *
	 * @param node
	 */
	@WrapperMember
	protected LocalVariableNode setNode(LocalVariableNode node) {
		this.node = node;
		WrapperRecycler.add(this, node);
		return node;
	}
	
	/**
	 * Getter allows for wrappee refinement.
	 *
	 * @return
	 */
	@WrapperMember
	protected LocalVariableNode getNode() {
		return node;
	}

	/**
	 * Creates a local variable.
	 *
	 * @param index
	 * @param name
	 * @param type
	 * @param start
	 * @param end
	 */
	public LocalVariableGen(int index, String name, Type type, InstructionHandle start, InstructionHandle end) {
		LabelNode startLabel = start == null ? null : start.ensureLabel();
		LabelNode endLabel = end == null ? null : end.ensureNextLabel();
		
		setNode(new LocalVariableNode(name, type.getType().getDescriptor(), null, startLabel, endLabel, index));

		if (null != start)
			start.addTargeter(this);
		if (null != end)
			end.addTargeter(this);
	}
	
	/**
	 * Gets the start index for this variable.
	 *
	 * @return
	 */
	public int getIndex() {
		return this.getNode().index;
	}
	
	/**
	 * Returns all labels that are targets of this instruction.
	 *
	 * @return
	 */
	@WrapperMember
	public LabelNode[] getTargetLabels() {
		LocalVariableNode n = (LocalVariableNode) getNode();
		return new LabelNode[] {n.start, n.end};
	}
}
